import time
import hmac
import hashlib
import base64
import urllib.parse
import requests
import json


# 钉钉机器人
class Robot:
    def __init__(self, secret, url):
        self.secret = secret
        self.url = url

    # 生成时间戳timestamp和签名数据sign用于钉钉机器人的请求
    def __gen_timestamp_and_sign(self):
        timestamp = str(round(time.time() * 1000))
        secret_enc = self.secret.encode('utf-8')
        string_to_sign = '{}\n{}'.format(timestamp, self.secret)
        string_to_sign_enc = string_to_sign.encode('utf-8')
        hmac_code = hmac.new(secret_enc, string_to_sign_enc, digestmod=hashlib.sha256).digest()
        sign = urllib.parse.quote_plus(base64.b64encode(hmac_code))
        return timestamp, sign

    # 发送钉钉机器人通知
    def send_msg(self, msg_data):
        # 从gen_timestamp_and_sign方法获取timestamp和sign
        timestamp, sign = self.__gen_timestamp_and_sign()
        # 机器人url
        robot_url = self.url
        # 拼接请求url
        url = '{0}&timestamp={1}&sign={2}'.format(robot_url, timestamp, sign)
        print(url)
        # 请求头
        headers = {'Content-Type': 'application/json'}
        # 发送请求
        ret = requests.post(url, headers=headers, data=json.dumps(msg_data), verify=False)
        # 判断请求结果
        ret_dict = ret.json()
        if ret_dict.get('errcode') == 0:
            print('消息发送成功')
        else:
            print('消息发送失败: {}'.format(ret_dict.get('errmsg')))


if __name__ == '__main__':
    # im测试群：im测试助手
    robot_info_default = {
        'secret': 'SECe0e8136eb7b81f712cb3b277cde794159c9ab24f3489b1d9f26629d79494403a',
        'webhook_url': 'https://oapi.dingtalk.com/robot/send?access_token=487648aceb8a9e499673307c9647c2332599'
                       'd0079a7a00c80ce5b1870b1a4b22'
    }
    r_secret = robot_info_default.get('secret')
    r_url = robot_info_default.get('webhook_url')

    # 默认消息内容
    data_default = {
        "msgtype": "markdown",
        "markdown": {
            "title": '测试',
            "text": '测试123'
        }
    }

    # 消息拼接示例
    # result_text = "### *{0}*\n\n" \
    #     "<font color=#C0C0C0>用例数：</font>{1}，" \
    #     "<font color=#C0C0C0>通过：</font><font color=#00FF00>{2}</font>，" \
    #     "<font color=#C0C0C0>失败：</font><font color=#FF0000>{3}</font>，" \
    #     "<font color=#C0C0C0>成功率：</font><font color=#00FF00>{4}%</font>\n" \
    #     "> #### [查看详情]({5})\n".format(msg_title, total, pass_num, fail_num, pass_rate, url)

    robot = Robot(r_secret, r_url)
    robot.send_msg(data_default)



import binascii
import os
import subprocess
import time

import allure
import requests
import wda

from utils.log import logger
from utils.config import conf
from core.ios.wda_server import WDAServer


# 重启wda的装饰器
def relaunch_wda(func):
    def wrapper(self, *args, **kwargs):
        return _inner_retry(self, func, *args, **kwargs)
    return wrapper


def _inner_retry(self, func, *args, **kwargs):
    count = 1
    while count < 4:
        try:
            return func(self, *args, **kwargs)
        except (requests.exceptions.ConnectionError, wda.WDAError, binascii.Error) as _:
            logger.info(f'重连wda第{count}次')
            ws = WDAServer(conf.get_name('device', 'serial_no'))
            ws.launch_wda()
            count = count + 1
    return func(self, *args, **kwargs)


class IosDriver(object):
    _instance = {}

    def __new__(cls, serial_no=None):
        if serial_no not in cls._instance:
            cls._instance[serial_no] = super().__new__(cls)
        return cls._instance[serial_no]

    def __init__(self, serial_no=None):
        if not serial_no:
            self.serial_no = conf.get_name('device', 'serial_no')
        else:
            self.serial_no = serial_no
        self.bundle_id = conf.get_name('app', 'pkg_name')
        self.d = wda.USBClient(self.serial_no, port=8100)

    def uninstall_app(self, bundle_id=None):
        if not bundle_id:
            bundle_id = self.bundle_id
        cmd = f'tidevice -u {self.serial_no} uninstall {bundle_id}'
        logger.info(f'卸载应用: {bundle_id}')
        output = subprocess.getoutput(cmd)
        if 'Complete' in output.split()[-1]:
            logger.info(f'{self.serial_no} 卸载应用{bundle_id} 成功')
            return
        else:
            logger.info(f'{self.serial_no} 卸载应用{bundle_id}失败，因为{output}')

    def install_app(self, ipa_url, is_new=False):
        if is_new:
            self.uninstall_app(self.bundle_id)
        cmd = f'tidevice -u {self.serial_no} install {ipa_url}'
        logger.info(f'安装应用: {ipa_url}')
        output = subprocess.getoutput(cmd)
        if 'Complete' in output.split()[-1]:
            logger.info(f'{self.serial_no} 安装应用{ipa_url} 成功')
            return
        else:
            logger.info(f'{self.serial_no} 安装应用{ipa_url}失败，因为{output}')

    @relaunch_wda
    def start_app(self, bundle_id=None):
        if not bundle_id:
            bundle_id = self.bundle_id
        try:
            logger.info(f'启动应用: {bundle_id}')
            self.d.app_start(bundle_id)
            logger.info(f'{self.serial_no} 成功启动APP成功: {bundle_id}')
        except requests.exceptions.ReadTimeout:
            logger.error(f'{self.serial_no} 成功启动APP失败: {bundle_id}')

    @relaunch_wda
    def force_start_app(self, bundle_id=None):
        if not bundle_id:
            bundle_id = self.bundle_id
        logger.info(f'强制启动应用: {bundle_id}')
        self.go_home()
        self.d.app_terminate(bundle_id)
        self.start_app(bundle_id)

    @relaunch_wda
    def stop_app(self, bundle_id=None):
        if not bundle_id:
            bundle_id = self.bundle_id
        logger.info(f'停止应用: {bundle_id}')
        self.d.app_terminate(bundle_id)

    @relaunch_wda
    def app_current(self):
        cur_apps = self.d.app_current()
        logger.info(f'获取运行中的app列表: {cur_apps}')
        return cur_apps

    @relaunch_wda
    def app_launch(self, bundle_id=None):
        if not bundle_id:
            bundle_id = self.bundle_id
        logger.info(f'将应用切到前台: {bundle_id}')
        self.d.app_launch(bundle_id)

    @relaunch_wda
    def back(self):
        logger.info('返回上一页')
        time.sleep(1)
        self.d.swipe(0, 100, 100, 100)

    @relaunch_wda
    def locked(self):
        logger.info('锁屏')
        self.d.locked()

    @relaunch_wda
    def unlock(self):
        logger.info('退出锁屏')
        if self.locked():
            self.d.unlock()

    @relaunch_wda
    def go_home(self):
        logger.info('返回手机主页')
        self.d.home()

    @relaunch_wda
    def send_keys(self, value):
        logger.info(f'输入: {value}')
        self.d.send_keys(value)

    @relaunch_wda
    def screenshot(self, img_path):
        logger.info(f'截图并保存至: {img_path}')
        im = self.d.screenshot()
        rgb_im = im.convert('RGB')
        rgb_im.save(img_path)

    @relaunch_wda
    def allure_shot(self, filename):
        logger.info(f'{filename}-截图')
        self.screenshot('tmp.png')
        allure.attach.file('tmp.png', attachment_type=allure.attachment_type.PNG, name=f'{filename}-截图')
        os.remove('tmp.png')
	
	@relaunch_wda
	def wait_shot(self, filename, timeout=3):
        time.sleep(timeout)
        self.allure_shot(filename)

    @relaunch_wda
    def get_ui_tree(self):
        page_source = self.d.source(accessible=False)
        logger.info(f'获取页面内容: \n{page_source}')
        return page_source

    @relaunch_wda
    def get_window_size(self):
        size = self.d.window_size()
        logger.info(f'获取屏幕尺寸: {size}')
        return size

    @relaunch_wda
    def tap(self, x, y):
        logger.info(f'点击坐标: ({x}, {y})')
        logger.info(f'{self.serial_no} Tap point ({x}, {y})')
        self.d.appium_settings({"snapshotMaxDepth": 0})
        self.d.tap(x, y)
        self.d.appium_settings({"snapshotMaxDepth": 50})
        time.sleep(1)

    @relaunch_wda
    def swipe(self, start_x, start_y, end_x, end_y, duration=0):
        logger.info(f'从坐标({start_x}, {start_y})滑动到({end_x}, {end_y})')
        logger.info(f'{self.serial_no} swipe from point ({start_x}, {start_y}) to ({end_x}, {end_y})')
        self.d.appium_settings({"snapshotMaxDepth": 2})
        self.d.swipe(int(start_x), int(start_y), int(end_x), int(end_y), duration)
        self.d.appium_settings({"snapshotMaxDepth": 50})
        time.sleep(2)

    @relaunch_wda
    def swipe_by_screen_percent(self, start_x_percent, start_y_percent, end_x_percent, end_y_percent, duration=0):
        logger.info(f'根据屏幕百分比进行滑动')
        w, h = self.get_window_size()
        start_x = w * start_x_percent
        start_y = h * start_y_percent
        end_x = w * end_x_percent
        end_y = h * end_y_percent
        self.swipe(start_x, start_y, end_x, end_y, duration=duration)

    @relaunch_wda
    def swipe_left(self, start_percent=1, end_percent=0.5):
        logger.info('往左边滑动')
        w, h = self.get_window_size()
        self.swipe(start_percent * (w - 1), h/2, end_percent * w, h/2)

    @relaunch_wda
    def swipe_right(self, start_percent=0.5, end_percent=1):
        logger.info('往右边滑动')
        w, h = self.get_window_size()
        self.swipe(start_percent * w, h / 2, end_percent * (w - 1), h / 2)

    @relaunch_wda
    def swipe_up(self, start_percent=0.8, end_percent=0.2):
        logger.info('往上边滑动')
        w, h = self.get_window_size()
        self.swipe(w / 2, start_percent * h, w / 2, end_percent * h)

    @relaunch_wda
    def swipe_down(self, start_percent=0.2, end_percent=0.8):
        logger.info('往下面滑动')
        w, h = self.get_window_size()
        self.swipe(w / 2, start_percent * h, w / 2, end_percent * h)





import os
import subprocess
from utils.log import logger


class ChromeServer:
    @staticmethod
    def launch_server(self):
        logger.info('启动chromedriver')
        p = subprocess.Popen(['chromedriver', '--port={}'.format(9515)])
        try:
            return_code = p.wait(5)
            raise subprocess.CalledProcessError(return_code, p)
        except Exception as e:
            code = e.__dict__.get('returncode')
            if code is None:
                logger.info('chromedriver 启动成功')
            else:
                print(f'chromedriver 启动失败: {e.__dict__}')

    @staticmethod
    def stop_server():
        logger.info('\n停止chromedriver')
        try:
            os.popen('taskkill /f /im chromedriver.exe')
        except Exception as e:
            logger.error(f'停止chromedriver失败: {str(e)}')
        else:
            logger.info('停止chromedriver服务成功')





import smtplib
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from email.header import Header
from email.utils import formataddr


# 邮件通知
class Mail:
    def __init__(self, host, user, password):
        self.host = host
        self.username = user
        self.password = password

    def send_mail(self, mail_data, receivers):
        print(f'向{receivers}发送邮件...')
        # 创建一个带附件的实例
        message = MIMEMultipart()
        message['From'] = Header(self.username)
        message['To'] = Header(",".join(receivers))
        message['Subject'] = Header(mail_data.get('title'), 'utf-8')

        # 邮件正文内容
        message.attach(MIMEText(mail_data.get('body'), 'plain', 'utf-8'))
        # 附件
        file_path = mail_data.get('file_path')
        if file_path:
            # 构造附件，传送当前目录下的文件
            att1 = MIMEText(open(file_path, 'r').read(), 'base64', 'utf-8')
            att1["Content-Type"] = 'application/octet-stream'
            # 这里的filename可以任意写，写什么名字，邮件中显示什么名字
            file_name = mail_data.get('file_name')
            att1["Content-Disposition"] = f'attachment; filename="{file_name}"'
            message.attach(att1)

        # 连接
        conn = smtplib.SMTP_SSL(self.host, 465)
        # 登录
        conn.login(self.username, self.password)
        # 发送邮件
        try:
            conn.sendmail(self.username, receivers, message.as_string())
        except Exception as e:
            print(f'发送失败: {str(e)}')
        else:
            print('发送成功')
        # 断开连接
        conn.quit()


# 初始化
mail = Mail('smtphm.qiye.163.com', 'kang.yang@qizhidao.com', 'WzQzd2019')


if __name__ == '__main__':
    # 邮件内容
    mail_data = {
        'title': '测试邮件',
        'body': '测试',
        'file_path': None,
        'file_name': None
    }
    # 收件人
    receiver_list = ['kang.yang@qizhidao.com', 'kang.yang@qizhidao.com']
    mail.send_mail(mail_data, receiver_list)





import os
import sys
from lxml import etree


def read_report(report_file) -> dict:
    try:
        html = etree.parse(report_file, etree.HTMLParser())
        total = html.xpath('//p[2]/text()')[0].split()[0]
        pass_num = html.xpath('//span[@class="passed"]/text()')[0].split()[0]
        fail_num = html.xpath('//span[@class="failed"]/text()')[0].split()[0]
        data = {'total': total, 'pass_num': pass_num, 'fail_num': fail_num}
        print(f'报告数据: {data}')
        return data
    except Exception as e:
        print(f'读取文件异常, 退出程序\n{str(e)}')
        sys.exit()
    finally:
        try:
            os.remove(report_file)
        except FileNotFoundError:
            print(f'{report_file} 不存在')


if __name__ == '__main__':
    file1 = 'report.html'
    read_report(file1)



import requests
from selenium import webdriver
from utils.log import logger
from utils.config import conf
from core.web.chrome_server import ChromeServer
from core.android.element import driver


# 重启chromedriver的装饰器
def relaunch(func):
    def wrapper(self, *args, **kwargs):
        try:
            return func(self, *args, **kwargs)
        except requests.exceptions.ConnectionError as _:
            logger.warning("chromedriver error, relaunch now.")
            ChromeServer.launch_server()
    return wrapper


class Driver(object):
    _instance = {}

    def __new__(cls, serial_no=None):
        if serial_no not in cls._instance:
            cls._instance[serial_no] = super().__new__(cls)
        return cls._instance[serial_no]

    def __init__(self, serial_no=None, pkg_name=None):
        if not serial_no:
            self.serial_no = conf.get_name('device', 'serial_no')
        else:
            self.serial_no = serial_no
        if not pkg_name:
            self.pkg_name = conf.get_name('app', 'pkg_name')

        logger.info(f'启动webdriver')
        options = webdriver.ChromeOptions()
        options.add_experimental_option('androidDeviceSerial', self.serial_no)
        options.add_experimental_option('androidPackage', self.pkg_name)
        options.add_experimental_option('androidUseRunningApp', True)
        options.add_experimental_option('androidProcess', self.pkg_name)
        self.d = webdriver.Chrome(options=options)
        self.d.set_page_load_timeout(10)

    @relaunch
    def back(self):
        logger.info('返回上一页')
        self.d.back()

    @relaunch
    def send_keys(self, value):
        logger.info(f'输入文本: {value}')
        driver.send_keys(value)

    @relaunch
    def switch_input(self):
        logger.info('切换输入法')
        driver.d.set_fastinput_ime(enable=False)

    @relaunch
    def screenshot(self, filename, timeout=3):
        driver.wait_shot(filename, timeout=timeout)

    @relaunch
    def get_ui_tree(self):
        page_source = self.d.page_source()
        logger.info(f'获取页面内容: \n{page_source}')
        return page_source

    @relaunch
    def get_windows(self):
        logger.info(f'获取当前打开的窗口列表')
        return self.d.window_handles

    @relaunch
    def switch_window(self, old_windows):
        logger.info('切换到最新的window')
        current_windows = self.get_windows()
        newest_window = [window for window in current_windows if window not in old_windows][0]
        self.d.switch_to.window(newest_window)

    @relaunch
    def close(self):
        logger.info('关闭webdriver')
        self.d.close()

    @relaunch
    def execute_js(self, script, element):
        logger.info(f'执行js脚本: \n{script}')
        self.d.execute_script(script, element)










import os
import time
import allure
import uiautomator2 as u2
from utils.log import logger
from utils.config import conf


class Driver(object):
    _instance = {}

    def __new__(cls, serial_no=None):
        if not serial_no:
            serial_no = conf.get_name('device', 'serial_no')
        if serial_no not in cls._instance:
            cls._instance[serial_no] = super().__new__(cls)
        return cls._instance[serial_no]

    def __init__(self, serial_no=None):
        if not serial_no:
            self.serial_no = conf.get_name('device', 'serial_no')
        else:
            self.serial_no = serial_no
        self.pkg_name = conf.get_name('app', 'pkg_name')
        self.d = u2.connect(self.serial_no)

    def uninstall_app(self, pkg_name=None):
        if not pkg_name:
            pkg_name = self.pkg_name
        logger.info(f'卸载应用: {pkg_name}')
        self.d.app_uninstall(pkg_name)

    def install_app(self, apk_path, is_new=False):
        if is_new:
            self.uninstall_app(self.pkg_name)
        logger.info(f'安装应用: {apk_path}')
        self.d.app_install(apk_path)

    def start_app(self, pkg_name=None):
        if not pkg_name:
            pkg_name = self.pkg_name
        logger.info(f'启动应用: {pkg_name}')
        self.d.app_start(pkg_name)

    def stop_app(self, pkg_name=None):
        if not pkg_name:
            pkg_name = self.pkg_name
        logger.info(f'退出应用: {pkg_name}')
        self.d.app_stop(pkg_name)

    def force_start_app(self, pkg_name=None):
        if not pkg_name:
            pkg_name = self.pkg_name
        logger.info(f'强制启动应用: {pkg_name}')
        self.d.app_start(pkg_name, stop=True)

    def go_home(self):
        time.sleep(1)
        logger.info('返回手机桌面')
        self.d.press('home')

    def back(self):
        time.sleep(1)
        logger.info('返回上一页')
        self.d.press('back')

    def swipe(self, x1, y1, x2, y2, duration=0):
        logger.info(f'从坐标({x1}, {y1})滑到坐标({x2}, {y2})')
        self.d.swipe(x1, y1, x2, y2, duration=duration)
        time.sleep(2)

    def swipe_left(self):
        logger.info('向左边滑动')
        self.d.swipe_ext('left')

    def swipe_right(self):
        logger.info('向右边滑动')
        self.d.swipe_ext('right')

    def swipe_up(self):
        logger.info('向上滑动')
        self.d.swipe_ext('up')

    def swipe_down(self):
        logger.info('向下滑动')
        self.d.swipe_ext('down')

    def tap(self, x, y):
        logger.info(f'点击坐标({x},{y})')
        self.d.click(x, y)
        time.sleep(1)

    def screenshot(self, img_path):
        logger.info(f'截屏并保存至: {img_path}')
        image = self.d.screenshot()
        image.save(img_path)

    def allure_shot(self, filename):
        logger.info(f'{filename}-截图')
        self.d.screenshot('tmp.png')
        allure.attach.file('tmp.png', attachment_type=allure.attachment_type.PNG, name=f'{filename}-截图')
        os.remove('tmp.png')
	
	def wait_shot(self, filename, timeout=3):
        time.sleep(timeout)
        self.allure_shot(filename)

    def send_keys(self, value):
        logger.info(f'输入: {value}')
        self.d.set_fastinput_ime(True)
        self.d.send_keys(value)
        self.d.set_fastinput_ime(False)
        self.d.send_action('search')

    def input_password(self, value):
        logger.info(f'输入密码: {value}')
        self.d(focused=True).set_text(value)

    def enter(self):
        logger.info('键盘点击enter')
        self.d.press('enter')

    def delete(self):
        logger.info('点击一次退格键')
        self.d.press('delete')

    def clear(self, num=10):
        logger.info('清空输入框: 通过点击10次退格键实现')
        for i in range(num):
            self.delete()

    def get_ui_tree(self):
        page_source = self.d.dump_hierarchy()
        logger.info(f'获取页面内容: \n{page_source}')
        return page_source


# 从config.ini文件中获取serial_no进行初始化
driver = Driver()
d = driver.d





from utils.log import logger
from core.android.driver import d, driver
from core.web.driver import relaunch
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.wait import WebDriverWait


# 安卓原生元素
class AdrElement:
    def __init__(self, rid=None, text=None, xpath=None, index=None):
        self.element_loc = {'resourceId': rid, 'text': text, 'xpath': xpath, 'index': index}
        self.element_loc = {k: v for k, v in self.element_loc.items() if v is not None}  # 去除为None的定位器
        if xpath:
            self.element = d.xpath(xpath)
        else:
            if rid and text:
                self.element = d(resourceId=rid, text=text)
            elif rid and not text:
                self.element = d(resourceId=rid)
            elif not rid and text:
                self.element = d(text=text)
            if index:
                self.element = self.element[index]

    def wait(self, timeout=10, shot_flag=True):
        if self.element.wait(timeout=timeout):
            return True
        else:
            if shot_flag:
                driver.allure_shot(f'元素:{self.element_loc}定位失败')
                raise AssertionError(f'元素:{self.element_loc}定位失败')
            else:
                return False

    def click(self, timeout=10):
        logger.info(f'点击元素: {self.element_loc}')
        if self.wait():
            self.element.click(timeout=timeout)
	
	# 确保点击后页面会跳转，前提是跳转后的页面没有该元素
    def click_ensure(self, timeout=5):
        self.element.click()
        count = 0
        while count < timeout and self.element.exists(1):
            self.element.click()
            time.sleep(1)
            count += 1
	
    def send_keys(self, content):
        logger.info(f'定位元素并输入{content}: {self.element_loc}')
        if self.wait():
            self.element.send_keys(content)

    def get_text(self):
        logger.info(f'获取元素文本: {self.element_loc}')
        if self.wait():
            return self.element.get_text()
        else:
            return None

    # 用于常见分支场景判断
    def exists(self, timeout=1):
        logger.info(f'判断元素是否存在: {self.element_loc}')
		status = self.wait(timeout=timeout, shot_flag=False)
		logger.info(status)
        return status



import os
import pathlib
import configparser

local_path = os.path.dirname(os.path.realpath(__file__))
root_path = os.path.dirname(local_path)


class Config:
    def __init__(self):
        self.conf_file_path = os.path.join(root_path, 'config.ini')
        config_file = pathlib.Path(self.conf_file_path)
        config_file.touch(exist_ok=True)
        self.cf = configparser.ConfigParser()
        self.cf.read(self.conf_file_path, encoding='utf-8')

    def get_name(self, module, key):
        if not self.cf.has_option(module, key):
            print('未找到该数据')
            value = None
        else:
            value = self.cf.get(module, key)
        return value

    def set_name(self, module, key, value):
        if not self.cf.has_section(module):
            self.cf.add_section(module)
        self.cf.set(module, key, value)
        with open(self.conf_file_path, 'w') as f:
            self.cf.write(f)


# 初始化
conf = Config()

if __name__ == '__main__':
    pass
    # conf.set_name('test', 'test1', '123')
    # print(conf.get_name('test', 'test'))




# -*- coding: utf-8 -*-
from setuptools import setup, find_packages

try:
    long_description = open("README.md", encoding='utf-8').read()
except IOError:
    long_description = ""

setup(
    name="qrunner",
    version="0.0.3",
    description="APP自动化测试框架",
    author="杨康",
    author_email="772840356@qq.com",
    platforms="Android,IOS",
    packages=find_packages(),
    long_description=long_description,
    classifiers=[
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.9",
    ],
    install_requires=['tidevice', 'facebook-wda', 'uiautomator2', 'selenium', 'pytest', 'pytest-html',
                      'pytest-rerunfailures', 'allure-pytest']
)



import random
import time


# 打印时间
def _print_time():
    print(time.strftime("%Y%m%d%H%M"))


# 获取时间间隔
def _get_time_interval():
    time_a = time.time()
    time.sleep(3)
    time_b = time.time()
    interval = round(time_b - time_a, 2)
    print(f'时间间隔: {interval}s')
    return interval


# 生成时间戳
def get_timestamp(n=13) -> str:
    timestamp = str(time.time()).replace('.', '')[:n]
    timestamp = timestamp.ljust(n, '6')
    print(f'时间戳: {timestamp}')
    return timestamp


# 通过时间戳生成文本
def get_name(content) -> str:
    non_report_str = f'{content}-{get_timestamp()}'
    print(f'不重复的字符串: {non_report_str}')
    return non_report_str


# 通过时间戳生成手机号
def get_phone() -> str:
    # 移动手机号前几位
    cm = [134, 135, 136, 137, 138, 139, 150, 151, 152, 157, 158, 159, 182, 183, 184, 187, 188, 147, 178, 1705]
    # 联通手机号前几位
    cu = [130, 131, 132, 155, 156, 185, 186, 145, 176, 1709]
    # 手机号前几位
    ct = [133, 153, 180, 181, 189, 177, 1700]
    phone = str(random.choice(cm + cu + ct)) + get_timestamp()
    phone = phone[:11]
    print(f'手机号: {phone}')
    return phone


if __name__ == '__main__':
    # get_timestamp()
    # get_name('张三')
    # get_phone()
    _get_time_interval()






import os
import time

from core.ios.driver import IosDriver
from core.ios.driver import relaunch_wda
from utils.log import logger
from utils.config import conf

serial_no = conf.get_name('device', 'serial_no')
driver = IosDriver(serial_no)
d = driver.d


class IosElement:
    def __init__(self, name=None, value=None, xpath=None, index=None, label=None, labelContains=None):
        self.element_loc = {
            'name': name,
            'value': value,
            'xpath': xpath,
            'label': label,
            'index': index
        }
        self.element_loc = {k: v for k, v in self.element_loc.items() if v is not None}  # 去除为None的定位器
        if xpath:
            self.element = d.xpath(xpath)
        elif name:
            self.element = d(name=name)
        elif value:
            self.element = d(value=value)
        elif label:
            self.element = d(label=label)
        elif labelContains:
            self.element = d(labelContains=labelContains)
        if index:
            self.element = self.element[index]

    @relaunch_wda
    def wait(self, timeout=10, shot_flag=True):
        if self.element.wait(timeout=timeout):
            return True
        else:
            if shot_flag:
                driver.allure_shot(f'元素:{self.element_loc}定位失败')
                raise AssertionError(f'元素:{self.element_loc}定位失败')

    @relaunch_wda
    def scroll(self):
        logger.info(f'滚动到元素: {self.element_loc}')
        if self.wait():
            self.element.scroll()

    @relaunch_wda
    def click(self, timeout=10):
        logger.info(f'点击元素: {self.element_loc}')
        if self.wait(timeout=timeout):
            self.element.click()

    @relaunch_wda
    def send_keys(self, content):
        logger.info(f'定位元素: {self.element_loc} ,并输入: {content}')
        if self.wait():
            self.element.clear_text()
            self.element.set_text(content)
            d(label='Done').click()

    @relaunch_wda
    def get_text(self):
        logger.info(f'获取元素文本: {self.element_loc}')
        if self.wait():
            return self.element.text
        else:
            return None

    @relaunch_wda
    def get_value(self):
        logger.info(f'获取元素value: {self.element_loc}')
        if self.wait():
            return self.element.value
        else:
            return None

    @relaunch_wda
    def exists(self, timeout=1):
        logger.info(f'判断元素是否存在: {self.element_loc}')
		status = self.wait(timeout=timeout, shot_flag=False)
		logger.info(status)
        return status




from utils.log import logger
from core.web.driver import relaunch
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.wait import WebDriverWait


# 安卓webview元素
class WebElement:
    def __init__(self, h5_driver, xpath=None, css=None, index=0):
        self.driver = h5_driver
        self.d = self.driver.d
        self.locator = None
        self.index = index
        if xpath:
            self.locator = (By.XPATH, xpath)
        if css:
            self.locator = (By.CSS_SELECTOR, css)

    @relaunch
    def wait(self, timeout=10):
        wait = WebDriverWait(self.d, timeout=timeout)
        try:
            wait.until(EC.presence_of_element_located(self.locator))
            return True
        except:
            self.driver.screenshot(f'元素定位失败: {self.locator}')
            # logger.info(f'页面内容: \n{self.driver.get_ui_tree()}')
            raise AssertionError(f'元素定位失败: {self.locator}')

    @relaunch
    def get_element(self):
        if self.wait():
            element = self.d.find_elements(self.locator[0], self.locator[1])[self.index]
            return element

    @relaunch
    def click(self):
        logger.info(f'点击元素: {self.locator}')
        self.get_element().click()

    @relaunch
    def get_text(self):
        logger.info(f'获取元素文案: {self.locator}')
        return self.get_element().text






import os
import subprocess

from utils.log import logger


class WDAServer:
    def __init__(self, serial_no):
        self.serial_no = serial_no

    def launch_wda(self):
        logger.info(f'{self.serial_no} 开始启动wda服务')
        # 通过tidevice命令启动代理服务
        cmd = 'tidevice -u {0} xctest'.format(self.serial_no)
        p = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        try:
            return_code = p.wait(5)
            raise subprocess.CalledProcessError(return_code, p)
        except Exception as e:
            code = e.__dict__.get('returncode')
            if code is None:
                logger.info(f'{self.serial_no} 启动wda服务成功.')
            else:
                logger.error(f'{self.serial_no} 启动wda服务失败: {e.__dict__}')

    @staticmethod
    def stop_wda():
        logger.info('\n停止wda服务')
        try:
            os.popen('taskkill /f /im tidevice.exe')
        except Exception as e:
            logger.error(f'停止wda服务失败: {str(e)}')
        else:
            logger.info('停止wda服务成功')

